----------------------------------------------------------------------------------
-- Company: NTNU
-- Engineer: Martin Lesund
-- 
-- Create Date: 03/28/2017 03:04:23 PM
-- Design Name: 
-- Module Name: SPI_master_tb - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: This is the testbench for the SPI_master.vhd
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-------------------------------------------------------------------------------
    -- entity remains blank for testbench
entity SPI_master_tb is
end SPI_master_tb;
-------------------------------------------------------------------------------

architecture Behavioral of SPI_master_tb is

--Component ports
   constant c_CLOCK_PERIOD  : time := 10ns;   -- 10ns = 100MHz 
   constant g_data_bus_width: integer := 16; --(16) number of bits to serialize
   constant g_num_of_slaves : integer := 1;  --Number of slaves
   constant g_divide_value  : integer := 1;   --o_SPI_sclk freq = i_clock/(2*g_divide_value)
   -------CPOL and CPHA MODES -----------------------------
   constant g_CPOL          : std_logic := '1';  
   constant g_CPHA          : std_logic := '1';
    
    signal i_clock          : std_logic := '0';
    signal i_reset_n        : std_logic;
    ----------------------------------------------
    signal i_enable       : std_logic;
    signal i_write_data   : std_logic_vector(g_data_bus_width-1 downto 0);
    signal o_done         : std_logic;
    signal o_read_data    : std_logic_vector(g_data_bus_width-1 downto 0);
    signal i_addr         : std_logic_vector(g_num_of_slaves-1 downto 0);     
   
       ----------------------------------------------
    signal o_SPI_sclk           : std_logic;
    signal o_SPI_ss_n           : std_logic_vector(g_num_of_slaves -1 downto 0);
    signal o_SPI_mosi           : std_logic;
    signal i_SPI_miso           : std_logic;
    
    -----------------------------------------------------
    signal COUNT_RISE : integer;
    signal COUNT_FALL : integer;
    signal TEST_MOSI  : std_logic_vector(g_data_bus_width-1 downto 0); -- tx data
    signal TEST_MISO  : std_logic_vector(g_data_bus_width-1 downto 0); --rx data
    signal r_addr     : integer;
    
    ------------------------------------------------------


  component SPI_master is  
  generic(
   g_data_bus_width : integer := 8;                              
   g_num_of_slaves  : integer;              
   g_divide_value   : integer;             
   g_CPOL           : std_logic;         
   g_CPHA           : std_logic          
  );
  port(
    i_clock         : in  std_logic;
    i_reset_n       : in  std_logic;
   
    i_enable       : in  std_logic;
    i_write_data   : in  std_logic_vector(g_data_bus_width-1 downto 0);    -- Transmitted Data to slaves
    o_done         : out std_logic;
    o_read_data    : out std_logic_vector(g_data_bus_width-1 downto 0);
    i_addr         : in  std_logic_vector(g_num_of_slaves-1 downto 0);     
    
    o_SPI_sclk        : out std_logic;
    o_SPI_ss_n        : out std_logic_vector(g_num_of_slaves -1 downto 0);
    o_SPI_mosi        : out std_logic;                         -- SPI master out, slave in, serial data line
    i_SPI_miso        : in  std_logic     
  );
  end component SPI_master;

begin
    --Component Instantiation
    -- Instantiate the Unit Under Test (UUT)
    UUT : SPI_master
    generic map(
        g_data_bus_width => g_data_bus_width,
        g_divide_value   => g_divide_value,
        g_CPOL           => g_CPOL,
        g_CPHA           => g_CPHA,
        g_num_of_slaves  => g_num_of_slaves
    )
    port map (
        i_clock        => i_clock,
        i_reset_n      => i_reset_n,
 -------------------------------------------------------------------------------------------------
        i_enable       => i_enable,   
        i_write_data   => i_write_data,
        o_done         => o_done,
        o_read_data    => o_read_data,
        i_addr         => i_addr,
      --  i_cont         => r_CONT,
-- SPI MASTER INTERFACE ----------------------------------------------------------------------------------    
        o_SPI_sclk      => o_SPI_sclk,
        o_SPI_ss_n      => o_SPI_ss_n,
        o_SPI_mosi      => o_SPI_mosi,
        i_SPI_miso      => i_SPI_miso      
    
    );
    

    -- CLOCK Generation
    p_CLK_GEN : process is
    begin
       i_clock <= '0';
        wait for c_CLOCK_PERIOD/2;
        i_clock <= '1';
        wait for c_CLOCK_PERIOD/2;
    end process p_CLK_GEN;

    i_reset_n <= '0', '1' after 163ns;
    i_addr <= std_logic_vector(to_unsigned(16#00#,g_num_of_slaves)); --Addressing slave
    r_addr <= to_integer(unsigned(i_addr));
    -- MAIN TESTING -----------------------
    -- FSM
    p_control : process(i_clock, i_reset_n)
        variable v_CONTROL  : unsigned(12 downto 0);
    begin
            if(i_reset_n = '0') then
            v_CONTROL  := (others => '0');
            i_enable <= '0';
            i_write_data  <= std_logic_vector(to_unsigned(16#92#,g_data_bus_width)); 
            
				  
        elsif(rising_edge(i_clock)) then
            v_CONTROL := v_CONTROL +1;
            if(v_CONTROL = 10) then -- 10
                i_enable  <= '1';
            else
                i_enable <= '0';
            end if;
            
            if(o_done = '1') then
                i_write_data <= std_logic_vector(unsigned(i_write_data)+1); 
                
            end if;
        end if;
     end process p_control;
     
     p_control_sclk : process(o_SPI_sclk)
     begin
        if(i_reset_n = '0') then
            TEST_MISO <= std_logic_vector(to_unsigned(16#C9#,g_data_bus_width)); 
            TEST_MOSI <= std_logic_vector(to_unsigned(16#00#,g_data_bus_width)); 
            i_SPI_miso <= '0';
            COUNT_RISE <= 0;
            COUNT_FALL <= 0;
        else
 ----------------------------------------------MODE11----------------------------------------------------           
            if (rising_edge(o_SPI_sclk) and g_CPOL= '1')then 
                if(o_SPI_ss_n(r_addr) = '0') then
                    TEST_MOSI <= TEST_MOSI(g_data_bus_width-2 downto 0)&o_SPI_mosi;
                    COUNT_RISE <= COUNT_RISE +1; 
                else
                    COUNT_RISE <= 0; 
                end if;
             end if;
             
            if (falling_edge(o_SPI_sclk) and g_CPOL= '1')then 
                if(o_SPI_ss_n(r_addr) = '0') then
                    TEST_MISO <= std_logic_vector(rotate_right(unsigned(TEST_MISO),1));
                    i_SPI_miso    <= TEST_MISO(g_data_bus_width-1); 
                    COUNT_FALL <= COUNT_FALL +1; 
                else
                    COUNT_FALL <= 0; 
                end if; 
            end if; 
 ----------------------------------------------MODE01----------------------------------------------------          
            if (falling_edge(o_SPI_sclk) and g_CPOL = '0')then  
                            if(o_SPI_ss_n(r_addr) = '0') then
                                TEST_MOSI <= TEST_MOSI(g_data_bus_width-2 downto 0)&o_SPI_mosi;
                                COUNT_FALL <= COUNT_FALL +1; 
                            else
                                COUNT_FALL <= 0; 
                            end if;
             end if;
                         
            if (rising_edge(o_SPI_sclk) and g_CPOL = '0')  then  
                            if(o_SPI_ss_n(r_addr) = '0') then
                                TEST_MISO <= std_logic_vector(rotate_right(unsigned(TEST_MISO),1));
                                i_SPI_miso    <= TEST_MISO(g_data_bus_width-1);
                                COUNT_RISE <= COUNT_RISE +1; 
                            else
                                COUNT_RISE <= 0; 
                            end if; 
             end if;
         end if;       
     end process p_control_sclk; 
     
end Behavioral;
